package tb3d;

import javax.swing.*;
import java.awt.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;
import java.io.RandomAccessFile;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.awt.event.*;


/**
 *  <p>Title: TreeBuilder3D</p>
 *  <p>Description: Interactive 3D viewer</p>
 *  <p>Copyright: Copyright (c) 2004</p>
 *  <p>Company: GSC BC Cancer Agency</p>
 *
 *@author     Peter Ruzanov
 *@version    1.0
 */

/*GPL
This file is part of TreeBuilder3D

TreeBuilder3D is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

TreeBuilder3D is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeBuilder3D; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

public class tb3d_controls extends JPanel {
    JScrollPane jsp01 = new JScrollPane();
    JList LibList = new JList();
    JButton OpenButton = new JButton();
    JButton SaveButton = new JButton();
    JButton DropButton = new JButton();
    JProgressBar ProgressBar = new JProgressBar();
    JPanel MiscPanel = new JPanel();
    ButtonGroup buttonGroup1 = new ButtonGroup();
    JRadioButton RadioButtonQuant = new JRadioButton();
    JRadioButton RadioButtonCorr = new JRadioButton();
    JRadioButton RadioButtonEuc = new JRadioButton();
    JRadioButton RadioButtonManh = new JRadioButton();
    JRadioButton RadioButtonCorr1 = new JRadioButton();
    JLabel Label_Highlight = new JLabel();
    JLabel MetLabel = new JLabel();
    GridLayout gridLayout1 = new GridLayout();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JFileChooser C_fileChooser = new JFileChooser();
    JCheckBox LogCBox = new JCheckBox();

    //Custom parameters:
    /**
     *  Description of the Field
     */
    protected static int[][] MATRIX;
    /**
     *  Description of the Field
     */
    protected static boolean FRESH = false;
// protected static boolean CRASH = false;
    /**
     *  Description of the Field
     */
    protected static Hashtable avergs;
    /**
     *  Description of the Field
     */
    protected static int StatType = 1;
    //Distance metrics: Quantitative 1, Correlation 2, Euclidean 3
    /**
     *  Description of the Field
     */
    protected static String[] NAMES;
    protected static String[] DESCS;

    private Hashtable libs;
    private Hashtable descs;
    private double[][] DMATRIX;
    private Hashtable TOTALS;
    private final double Log10 = Math.log(10);
    GridLayout gridLayout2 = new GridLayout();
    GridBagLayout gridBagLayout2 = new GridBagLayout();



    /**
     *  Constructor for the tb3d_controls object
     */
    public tb3d_controls() {
	try {
	    jbInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    /**
     *  Description of the Method
     *
     *@exception  Exception  Description of the Exception
     */
    private void jbInit() throws Exception {
	this.setLayout(gridBagLayout1);
	OpenButton.setText("Load Data");
	OpenButton.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OpenButton_actionPerformed(e);
		}
	    });
	SaveButton.setText("Save");
	SaveButton.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    SaveButton_actionPerformed(e);
		}
	    });

	DropButton.setText(">Drop");
	DropButton.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    DropButton_actionPerformed(e);
		}
	    });
	ProgressBar.setEnabled(true);
	ProgressBar.setForeground(new Color(153, 152, 220));
	ProgressBar.setVerifyInputWhenFocusTarget(false);
	ProgressBar.setString("0%");
	ProgressBar.setStringPainted(true);
	RadioButtonQuant.setAlignmentX((float) 0.1);
	RadioButtonQuant.setDoubleBuffered(false);
	RadioButtonQuant.setMaximumSize(new Dimension(98, 24));
	RadioButtonQuant.setMinimumSize(new Dimension(98, 24));
	RadioButtonQuant.setPreferredSize(new Dimension(98, 24));
	RadioButtonQuant.setFocusPainted(false);
	RadioButtonQuant.setHorizontalAlignment(SwingConstants.LEADING);
	RadioButtonQuant.setMargin(new Insets(0, 0, 0, 0));
	RadioButtonQuant.setSelected(true);
	RadioButtonQuant.setText("Quantitative");
	RadioButtonQuant.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    RadioButtonQuant_actionPerformed(e);
		}
	    });
	RadioButtonCorr.setAlignmentX((float) 0.1);
	RadioButtonCorr.setMaximumSize(new Dimension(124, 24));
	RadioButtonCorr.setMinimumSize(new Dimension(124, 24));
	RadioButtonCorr.setPreferredSize(new Dimension(124, 24));
	RadioButtonCorr.setRequestFocusEnabled(true);
	RadioButtonCorr.setActionCommand("Correlation");
	RadioButtonCorr.setFocusPainted(false);
	RadioButtonCorr.setHorizontalAlignment(SwingConstants.LEADING);
	RadioButtonCorr.setHorizontalTextPosition(SwingConstants.TRAILING);
	RadioButtonCorr.setMargin(new Insets(0, 0, 0, 0));
	RadioButtonCorr.setRolloverEnabled(false);
	RadioButtonCorr.setText("Pearson Squared");
	RadioButtonCorr.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    RadioButtonCorr_actionPerformed(e);
		}
	    });
	RadioButtonEuc.setForeground(Color.black);
	RadioButtonEuc.setAlignmentX((float) 0.1);
	RadioButtonEuc.setAlignmentY((float) 0.5);
	RadioButtonEuc.setMaximumSize(new Dimension(108, 24));
	RadioButtonEuc.setMinimumSize(new Dimension(108, 24));
	RadioButtonEuc.setPreferredSize(new Dimension(108, 24));
	RadioButtonEuc.setFocusPainted(false);
	RadioButtonEuc.setHorizontalAlignment(SwingConstants.LEADING);
	RadioButtonEuc.setHorizontalTextPosition(SwingConstants.TRAILING);
	RadioButtonEuc.setMargin(new Insets(0, 0, 0, 0));
	RadioButtonEuc.setMnemonic('0');
	RadioButtonEuc.setText("Euclidean");
	RadioButtonEuc.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    RadioButtonEuc_actionPerformed(e);
		}
	    });
	Label_Highlight.setText("Datasets:");
	MetLabel.setText("Metric:");
	gridLayout1.setColumns(0);
	gridLayout1.setRows(3);
	LogCBox.setText("Log scale");
	LogCBox.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    LogCBox_actionPerformed(e);
		}
	    });
	MiscPanel.setLayout(gridBagLayout2);
	MiscPanel.setBorder(BorderFactory.createEtchedBorder());
	MiscPanel.setMaximumSize(new Dimension(209, 34));
	MiscPanel.setMinimumSize(new Dimension(209, 34));
	MiscPanel.setPreferredSize(new Dimension(209, 34));
	this.setMaximumSize(new Dimension(2147483647, 450));
	this.setMinimumSize(new Dimension(456, 450));
	this.setPreferredSize(new Dimension(693, 450));
	RadioButtonManh.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    RadioButtonManh_actionPerformed(e);
		}
	    });
	RadioButtonManh.setText("Manhattan");
	RadioButtonManh.setRolloverEnabled(false);
	RadioButtonManh.setMargin(new Insets(0, 0, 0, 0));
	RadioButtonManh.setHorizontalTextPosition(SwingConstants.TRAILING);
	RadioButtonManh.setFocusPainted(false);
	RadioButtonManh.setHorizontalAlignment(SwingConstants.LEADING);
	RadioButtonManh.setActionCommand("Manhatten");
	RadioButtonManh.setRequestFocusEnabled(true);
	RadioButtonManh.setPreferredSize(new Dimension(124, 24));
	RadioButtonManh.setMinimumSize(new Dimension(124, 24));
	RadioButtonManh.setMaximumSize(new Dimension(124, 24));
	RadioButtonManh.setAlignmentX((float) 0.1);
	RadioButtonCorr1.setAlignmentX((float) 0.0);
	RadioButtonCorr1.setMaximumSize(new Dimension(148, 24));
	RadioButtonCorr1.setMinimumSize(new Dimension(148, 24));
	RadioButtonCorr1.setPreferredSize(new Dimension(148, 24));
	RadioButtonCorr1.setActionCommand("CorrUncentered");
	RadioButtonCorr1.setMargin(new Insets(0, 0, 0, 0));
	RadioButtonCorr1.setText("Pearson Uncentered");
	RadioButtonCorr1.addActionListener(
	    new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    RadioButtonCorr1_actionPerformed(e);
		}
	    });
	MiscPanel.add(RadioButtonCorr, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 34), 0, 0));
	MiscPanel.add(RadioButtonQuant, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 47), 0, 0));
	MiscPanel.add(RadioButtonManh, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 24), 0, 0));
	MiscPanel.add(RadioButtonEuc, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 32), 0, 0));
	MiscPanel.add(RadioButtonCorr1, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

	C_fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	jsp01.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

	this.add(Label_Highlight, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(19, 10, 0, 71), 16, 0));
	this.add(OpenButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
		, GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE, new Insets(13, 10, 0, 0), 21, 0));
	this.add(SaveButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
		, GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE, new Insets(13, 10, 0, 39), 26, 0));
	this.add(DropButton, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
		, GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE, new Insets(13, 10, 0, 0), 13, 0));
	this.add(jsp01, new GridBagConstraints(0, 1, 1, 5, 1.0, 1.0
		, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(13, 10, 19, 10), 149, 175));
	this.add(MetLabel, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
		, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(12, 21, 0, 40), 4, 0));
	this.add(ProgressBar, new GridBagConstraints(1, 5, 2, 1, 0.0, 0.0
		, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 10, 10), 0, 3));
	this.add(LogCBox, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
		, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));
	this.add(MiscPanel, new GridBagConstraints(1, 3, 2, 1, 0.0, 1.0
		, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(20, 10, 43, 18), 11, 463));
	jsp01.getViewport().add(LibList, null);
	buttonGroup1.add(RadioButtonQuant);
	buttonGroup1.add(RadioButtonCorr);
	buttonGroup1.add(RadioButtonEuc);
	buttonGroup1.add(RadioButtonManh);
	buttonGroup1.add(RadioButtonCorr1);
    }


    /**
     *  Function for reading in file and adding to the Hashtabe libs Here we
     *  read two kinds of formats: id either with or without count
     *
     *@param  FileSet  Description of the Parameter
     */
    private void AddLibrary(final File[] FileSet) {
	OpenButton.setEnabled(false);
	DropButton.setEnabled(false);
	SaveButton.setEnabled(false);
	RadioButtonCorr.setEnabled(false);
	RadioButtonEuc.setEnabled(false);
	RadioButtonQuant.setEnabled(false);
	RadioButtonManh.setEnabled(false);
	RadioButtonCorr1.setEnabled(false);

	final SwingWorker LibLoader =
	    new SwingWorker() {
		String Line;


		public Object construct() {

		    ProgressBar.setMaximum(FileSet.length);
		    for (int i = 0; i < FileSet.length; i++) {
			String name = FileSet[i].getName();
			String desc = name;
			ProgressBar.setString(name + " loading...");
			ProgressBar.setValue(i);
			if (name.indexOf(".") > 0) {
			    name = name.substring(0, name.indexOf("."));
			    desc = name;
			}
			RandomAccessFile f;
			try {
			    f = new RandomAccessFile(FileSet[i].getPath(), "r");
			} catch (FileNotFoundException nfe) {
			    ProgressBar.setString("No such file:" + name);
			    continue;
			}

			//String Line;

			int entries = 0;
			//int total = 0;
			int ctotal = 0;
			//this is read from a file
			boolean checked = false;

			Hashtable temp_gene = new Hashtable();
			//for holding info on this library

			try {
			    while ((Line = f.readLine()) != null) {
				//---------------------BLOCK FOR READING TOTAL TAGS INFO---------------

				if (!checked) {
				    try {
					if (Line.charAt(0) == '>' || Line.charAt(0) == '^' || Line.charAt(0) == '!') {
					    if (Line.indexOf("TOTAL") > 0) {
						String[] checker = Line.split("\t");
						if(checker[1] != "")
						 ctotal = Integer.parseInt(checker[1]);
						continue;
					    }
					    if (Line.indexOf("SAMPLE") > 0) {
						name = Line.substring(Line.indexOf("=") + 1);
						continue;
					    }
					    if (Line.indexOf("DESCRIPTION") > 0) {
					      String[] checker = Line.split("\t");
						if(checker[1] != "")
						 desc = checker[1];
					       continue;
					    }
					    if (Line.indexOf("Sample_title") > 0){
						desc = Line.substring(Line.indexOf("=") + 2);
						continue;
					    }
					continue;
					}
					if (Line.charAt(0) == '#')
					  continue;
					checked = true;
				    } catch (Exception ce) {
					ctotal = 0;
					checked = true;
					continue;
				    }
				}
				//---------------------BLOCK FOR READING TOTAL TAGS INFO ENDS------------

				String[] cols = Line.split("\t");
                                if(cols.length < 2)
                                  continue;
				double count = 1;
				try{
				    if (cols[1] != null) {
					count = Double.parseDouble(cols[1]);
				    }
				} catch (java.lang.NumberFormatException nfe) {
				    continue;
				} catch (Exception ex) {
				    temp_gene.clear();
				    entries = 0;
				    OpenButton.setEnabled(true);
				    DropButton.setEnabled(true);
				    SaveButton.setEnabled(true);
				    RadioButtonCorr.setEnabled(true);
				    RadioButtonEuc.setEnabled(true);
				    RadioButtonQuant.setEnabled(true);
				    RadioButtonManh.setEnabled(true);
				    RadioButtonCorr1.setEnabled(true);
				    break;
				}

				if (!temp_gene.containsKey(cols[0])) {
				    entries++;
				    temp_gene.put(cols[0], new Gene(cols[0], count));
				    //total += count;
				} else {
				    Gene current = (Gene) temp_gene.get(cols[0]);
				    current.count += count;
				    //total += count;
				    temp_gene.put(cols[0], current);
				}

			    }
			} catch (Exception e) {
			    ProgressBar.setString("Reading Error");
			    continue;
			}
			if (entries > 1) {
			    Gene[] temp = new Gene[entries];
			    Enumeration g = temp_gene.elements();
			    int pos = 0;
			    while (g.hasMoreElements()) {
				temp[pos] = (Gene) g.nextElement();
				pos++;
			    }
			    temp_gene.clear();
			    if (libs == null) {
				libs = new Hashtable();
				descs = new Hashtable();
			    }
			    if (TOTALS == null) {
				TOTALS = new Hashtable();
			    }
			    if (avergs != null) {
				avergs.clear();
			    }
			    avergs = null;
			    libs.put(name, temp);
			    if(desc == null)
			      desc = name;
			    descs.put(name, desc);
			    if (ctotal == 0) {
				TOTALS.put(name, "0");
			    } else {
				TOTALS.put(name, "" + ctotal);
			    }
			}

		    }
		    return Line;
		}


		public void finished() {

		    try {
			FRESH = true;
			UpdateParams();
			ProgressBar.setValue(0);
			ProgressBar.setString("");

			if (libs.size() == 1) {
			    OpenButton.setEnabled(true);
			    DropButton.setEnabled(true);
			    SaveButton.setEnabled(true);
			    RadioButtonCorr.setEnabled(true);
			    RadioButtonEuc.setEnabled(true);
			    RadioButtonQuant.setEnabled(true);
			    return;
			}

			RecalcDist();
		    } catch (Exception e) {
			ProgressBar.setString("Reading Error");
		    }
		}
	    };
	LibLoader.start();
    }



    /**
     *  Recalculation of distance matrix - the most important function
     */
    private void RecalcDist() {

	if (this.libs == null || this.libs.size() < 2) {
	    return;
	}

	if (OpenButton.isEnabled()) {
	    OpenButton.setEnabled(false);
	    DropButton.setEnabled(false);
	    SaveButton.setEnabled(false);
	    RadioButtonCorr.setEnabled(false);
	    RadioButtonEuc.setEnabled(false);
	    RadioButtonQuant.setEnabled(false);
	    RadioButtonManh.setEnabled(false);
	    RadioButtonCorr1.setEnabled(false);
	}

	final SwingWorker Distancer =
	    new SwingWorker() {
		double max = 0;
		//positive max


		public Object construct() {
		    MATRIX = new int[NAMES.length][NAMES.length];
		    DMATRIX = new double[NAMES.length][NAMES.length];
		    ProgressBar.setMaximum((NAMES.length * (NAMES.length - 1)) / 2);
		    ProgressBar.setString("Calculating distance");
		    int progr = 0;
		    for (int i = 0; i < NAMES.length; i++) {
			for (int j = i + 1; j < NAMES.length; j++) {
			    ProgressBar.setValue(progr);
			    ProgressBar.setString(progr + 1 + "of" + ProgressBar.getMaximum());
			    String lib1 = NAMES[i];
			    String lib2 = NAMES[j];
			    DMATRIX[i][j] = tb3d_stat.getDist(new Mix(lib1, lib2), StatType);
			    progr++;

			    max = Math.max(Math.abs(DMATRIX[i][j]), max);
			}
		    }
		    return DMATRIX;
		}


		public void finished() {
		    ProgressBar.setValue(0);
		    ProgressBar.setString("");
		    double coeff = 500 / max;
		    if (LogCBox.isSelected()) {
			coeff = 500 / ((Math.log(1 + max)) / Log10);
		    }

		    for (int i = 0; i < NAMES.length; i++) {
			for (int j = i + 1; j < NAMES.length; j++) {
			    if (LogCBox.isSelected()) {
				MATRIX[i][j] = (int) (((Math.log(Math.abs(DMATRIX[i][j]) + 1)) / Log10) * coeff);
			    } else {
				MATRIX[i][j] = (int) ((Math.abs(DMATRIX[i][j])) * coeff);
				if (MATRIX[i][j] == 0) {
				    MATRIX[i][j] = 1;
				}
			    }
			}
		    }
		    FRESH = true;
		    OpenButton.setEnabled(true);
		    DropButton.setEnabled(true);
		    SaveButton.setEnabled(true);
		    RadioButtonCorr.setEnabled(true);
		    RadioButtonEuc.setEnabled(true);
		    RadioButtonQuant.setEnabled(true);
		    RadioButtonManh.setEnabled(true);
		    RadioButtonCorr1.setEnabled(true);

		}
	    };
	Distancer.start();
    }


    /**
     *  Updates NAMES array and list of libraries in the interface
     */

    private void UpdateParams() {
	if (this.libs == null) {
	    return;
	}
	try {
	    Object[] new_names = libs.keySet().toArray();

	    NAMES = new String[new_names.length];
	    DESCS = new String[new_names.length];
	    for (int i = 0; i < NAMES.length; i++) {
		NAMES[i] = new_names[i].toString();
		DESCS[i] = descs.get(NAMES[i]).toString();
	    }

	    this.LibList.setListData(NAMES);

	} catch (Exception e) {
	    return;
	}

    }


    /**
     *  Saving MATRIX array into a file
     *
     *@param  fileName  Name of a file for writing the MATRIX data
     */

    private void SaveData(String fileName) {
	if (this.MATRIX == null || this.MATRIX[0].length < 2) {
	    ProgressBar.setString("No Data to Save");
	    return;
	}

	try {
	    File tester = new File(fileName);
	    if (!tester.createNewFile()) {
		Component cont = this.getRootPane().getParent();
		int result = JOptionPane.showConfirmDialog(cont,
			"File exists, Overwrite?",
			"WARNING",
			JOptionPane.WARNING_MESSAGE,
			JOptionPane.YES_NO_OPTION);
		if (result == 2) {
		    return;
		} else {
		    tester.delete();
		    tester = new File(fileName);
		    tester.createNewFile();
		}
	    }
	    RandomAccessFile Output = new RandomAccessFile(tester, "rw");
	    //Put the Header first
	    Output.writeBytes(">NAMES\t");
	    for (int i = 0; i < NAMES.length; i++) {
		Output.writeBytes("\t" + NAMES[i]);
	    }
            //Write the descriptions for the libraries
            Output.writeBytes("\n");
            Output.writeBytes(">DESCS\t");
            for (int d = 0; d < DESCS.length; d++) {
                Output.writeBytes("\t" + DESCS[d]);
            }
	    Output.writeBytes("\n");
	    for (int j = 0; j < NAMES.length; j++) {
		Output.writeBytes(NAMES[j] + "\t");
		for (int k = 0; k < NAMES.length; k++) {
		    if (j == k) {
			Output.writeBytes("\tX");
		    } else {
			String output = "" + DMATRIX[j][k];
			if(output.indexOf('.') > 0)
			      output = (output.indexOf('.') + 3 > output.length()) ? output : output.substring(0,output.indexOf('.') + 3);
			Output.writeBytes("\t" + output);
		    }

		}
		Output.writeBytes("\n");
	    }
	    Output.close();
	} catch (FileNotFoundException e) {
	    e.printStackTrace();

	} catch (IOException e) {
	    e.printStackTrace();
	}

    }


    /**
     *  Service class for holding two pieces of information - id and quantity
     *  value for gene
     *
     *@author     pruzanov
     *@created    April 14, 2004
     */
    protected class Gene {
	/**
	 *  Constructor for the Gene object
	 *
	 *@param  key  Description of the Parameter
	 *@param  cnt  Description of the Parameter
	 */
	protected Gene(String key, double cnt) {
	    this.id = key;
	    this.count = cnt;
	}


	/**
	 *  Constructor for the Gene object
	 *
	 *@param  key  Description of the Parameter
	 *@param  cnt  Description of the Parameter
	 *@param  al   Description of the Parameter
	 */
	protected Gene(String key, double cnt, double al) {
	    this.id = key;
	    this.count = cnt;
	    this.alt = al;
	}


	/**
	 *  Description of the Field
	 */
	protected String id;
	/**
	 *  Description of the Field
	 */
	protected double count;
	/**
	 *  Description of the Field
	 */
	protected double alt;
    }


    /**
     *  Service class for holding two pieces of information - id and quantity
     *  value for gene
     *
     *@author     pruzanov
     *@created    April 14, 2004
     */

    protected class Mix {
	/**
	 *  Description of the Field
	 */
	protected Hashtable genes;
	/**
	 *  Description of the Field
	 */
	protected int Atotal;
	/**
	 *  Description of the Field
	 */
	protected int Btotal;


	/**
	 *  Constructor for the Mix object
	 *
	 *@param  AA  Description of the Parameter
	 *@param  BB  Description of the Parameter
	 */
	protected Mix(String AA, String BB) {
	    genes = new Hashtable();
	    Atotal = Integer.parseInt(TOTALS.get(AA).toString());
	    Btotal = Integer.parseInt(TOTALS.get(BB).toString());
	    Gene[] A = (Gene[]) libs.get(AA);
	    Gene[] B = (Gene[]) libs.get(BB);

	    for (int i = 0; i < A.length; i++) {
		genes.put(A[i].id, new Gene(A[i].id, A[i].count, 0.5));
	    }
	    for (int j = 0; j < B.length; j++) {
		if (genes.containsKey(B[j].id)) {
		    double old = ((Gene) genes.get(B[j].id)).count;
		    genes.put(B[j].id, new Gene(B[j].id, old, B[j].count));
		} else {
		    genes.put(B[j].id, new Gene(B[j].id, 0.5, B[j].count));
		}
	    }
	}
    }


//***************
//--------Events:
//***************
    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void OpenButton_actionPerformed(ActionEvent e) {
	C_fileChooser.setMultiSelectionEnabled(true);
	C_fileChooser.setDialogTitle("Open File");
	if (C_fileChooser.APPROVE_OPTION == C_fileChooser.showOpenDialog(this)) {
	    AddLibrary(C_fileChooser.getSelectedFiles());

	}
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void DropButton_actionPerformed(ActionEvent e) {
	if (libs == null) {
	    return;
	}
	Object[] drops = LibList.getSelectedValues();
	for (int o = 0; o < drops.length; o++) {
	    String drop = drops[o].toString();
	    if (drop != null && libs.containsKey(drop)) {
		libs.remove(drop);
	    }
	    if (avergs != null) {
		avergs.clear();
	    }
	    avergs = null;
	}

	UpdateParams();
	RecalcDist();
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void RadioButtonQuant_actionPerformed(ActionEvent e) {
	this.StatType = 1;
	RecalcDist();
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void RadioButtonCorr_actionPerformed(ActionEvent e) {
	this.StatType = 2;
	RecalcDist();
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void RadioButtonEuc_actionPerformed(ActionEvent e) {
	this.StatType = 3;
	RecalcDist();
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void SaveButton_actionPerformed(ActionEvent e) {
	C_fileChooser.setMultiSelectionEnabled(false);
	C_fileChooser.setDialogTitle("Save File");

	if (C_fileChooser.APPROVE_OPTION == C_fileChooser.showSaveDialog(this)) {
	    File out = C_fileChooser.getSelectedFile();
	    String path = out.getPath();
	    SaveData(path);
	}
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void LogCBox_actionPerformed(ActionEvent e) {
	double max = 0;
	double coeff = 1;

	if (this.DMATRIX == null) {
	    return;
	}

	for (int i = 0; i < DMATRIX[0].length; i++) {
	    for (int j = i + 1; j < DMATRIX[0].length; j++) {

		if (Math.abs(DMATRIX[i][j]) > max) {
		    max = Math.abs(DMATRIX[i][j]);
		}
	    }
	}

	if (this.LogCBox.isSelected()) {
	    coeff = 500 / ((Math.log(1 + max)) / Log10);
	} else {
	    coeff = 500 / max;
	}
	for (int i = 0; i < NAMES.length; i++) {
	    for (int j = i + 1; j < NAMES.length; j++) {
		if (this.LogCBox.isSelected()) {
		    MATRIX[i][j] = (int) (((Math.log(Math.abs(DMATRIX[i][j]) + 1)) / Log10) * coeff);
		} else {
		    MATRIX[i][j] = (int) ((Math.abs(DMATRIX[i][j])) * coeff);
		}
		if (MATRIX[i][j] == 0) {
		    MATRIX[i][j] = 1;
		}
	    }
	}

	FRESH = true;
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void RadioButtonManh_actionPerformed(ActionEvent e) {
	this.StatType = 5;
	RecalcDist();
    }


    /**
     *  Description of the Method
     *
     *@param  e  Description of the Parameter
     */
    void RadioButtonCorr1_actionPerformed(ActionEvent e) {
	this.StatType = 4;
	RecalcDist();
    }
}
